@@ -38,7 +38,13 @@ function Debugger(sel) {
             return options.inverse(this);
         }
     });
+    Handlebars.registerPartial('breakpoint-condition-template', $('#breakpoint-condition-template').html() );
+    Handlebars.registerPartial('breakpoint-action-template', $('#breakpoint-action-template').html() );
+    Handlebars.registerPartial('breakpoint-right-click-template', $('#breakpoint-right-click-template').html() );
     this.breakpointRightClickMenuTemplate = Handlebars.compile( $('#breakpoint-right-click-template').html() );
+    this.breakpointConditionTemplate = Handlebars.compile( $('#breakpoint-condition-template').html() );
+    this.breakpointActionTemplate = Handlebars.compile( $('#breakpoint-action-template').html() );
+    this.breakpointListItemTemplate = Handlebars.compile( $('#breakpoint-list-item-template').html() );
 
     // The step in, over, run buttons
     this.controlButtons = $('.control-button');
@@ -72,13 +78,41 @@ function Debugger(sel) {
                     this.breakableLineRightClicked.bind(this));
         this.elt.on('click', '.breakpoint-condition,.action',
                     this.editBreakpointCondition.bind(this));
+        this.elt.on('change', '.toggle-breakpoint,.toggle-action',
+                    this.toggleBreakpointOrAction.bind(this));
+        this.elt.on('click', '.remove-breakpoint',
+                    this.removeBreakpointFromList.bind(this));
+        this.elt.on('click', '.breakpoint-goto',
+                    this.scrollToBreakpoint.bind(this));
         this.elt.on('hover', '.popup-perl-var',
                     this.popoverPerlVar.bind(this));
 
+        $('#breakpoint-container-handle').click(function(e) {
+            var breakpointPane = $('#breakpoint-container'),
+                icon = $('#breakpoint-container-handle-icon');
+            if (breakpointPane.hasClass('extended')) {
+                breakpointPane.animate(
+                    { width: 0 },
+                    'fast',
+                    function() {
+                        breakpointPane.removeClass('extended');
+                        icon.removeClass('icon-chevron-left').addClass('icon-chevron-right');
+                    });
+            } else {
+                breakpointPane.addClass('extended');
+                breakpointPane.animate(
+                    { width: $('#side-tray').width() },
+                    'fast',
+                    function() {
+                        icon.removeClass('icon-chevron-right').addClass('icon-chevron-left');
+                    });
+            }
+        });
+
         this.elt.on('click', '.current-sub-and-args', function(e) {
             var codeTable = $(e.currentTarget).parent().find('.program-code'),
                 activeLine = codeTable.find('.active'),
-                activeLineno = parseInt(activeLine.attr('data-line'));
+                activeLineno = parseInt(activeLine.attr('data-lineno'));
 
             dbg.setCurrentLineForCodeTable(codeTable, activeLineno);
 
@@ -125,7 +159,8 @@ function Debugger(sel) {
                 var ghostDivider = $('#ghost-divider'),
                     width = $(window).width() - ghostDivider.offset().left;
                 $('#columnator').css('padding-right', width);
-                dbg.watchDiv.css({  width: width-1,
+                $('#drag-divider').next()
+                            .css({  width: width-1,
                                     'margin-right': 0-width });
                 ghostDivider.remove();
                 $(document).unbind('mousemove');
@@ -172,7 +207,7 @@ function Debugger(sel) {
     this.breakableLineClicked = function(e) {
         var $elt = $(e.target),
             filename = $elt.closest('.program-code').attr('data-filename'),
-            lineno = $elt.closest('.code-line').attr('data-line'),
+            lineno = $elt.closest('.code-line').attr('data-lineno'),
             bp = this.breakpointManager.get(filename, lineno),
             requestData = { f: filename, l: lineno };
 
@@ -188,25 +223,103 @@ function Debugger(sel) {
     };
 
     // Triggered by the breakpoint manager whenever a breakpoint changes
+    // Update the breakpoint pane list
+    this.updateBreakpointList = function(filename, bp_hash) {
+        var breakpointsList = $('#breakpoints-list'),
+            breakpointListItem,
+            bpFileItems = breakpointsList.children(),
+            bpFileItem = breakpointsList.find('[data-filename="'+filename+'"] ol'),
+            isDeleteBreakpoint;
+
+        for (line in bp_hash) {
+            isDeleteBreakpoint = ! (bp_hash[line].condition || bp_hash[line].action);
+
+            breakpointListItem = breakpointsList.find('[data-filename="' + filename
+                                                        + '"][data-lineno="' + line + '"]'),
+            newBpListItem = $(dbg.breakpointListItemTemplate(
+                                    $.extend({filename : filename, lineno: line}, bp_hash[line])));
+
+            if (isDeleteBreakpoint && breakpointListItem.length) {
+                breakpointListItem.remove();
+
+            } else if (breakpointListItem.length) {
+                // already exists
+                breakpointListItem.empty().html(newBpListItem.html());
+            } else {
+                // Make it new
+                // first, see if we're already listing this file
+                var nextFileItem,
+                    i;
+
+                if (bpFileItem.length === 0) {
+                    // Didn't find this file listed; add this file in the appropriate place
+                    for (i = 0; i < bpFileItems.length; i++) {
+                        if (bpFileItems[i].attributes.getNamedItem('data-filename').value > filename) {
+                            nextFileItem = bpFileItems[i];
+                            break;
+                        }
+                    }
+                    if (nextFileItem) {
+                        bpFileItem = $('<li data-filename="'+filename+'"><span><small>'
+                                                +filename+'</small></span><ol></ol></li>')
+                                        .insertBefore(nextFileItem);
+                    } else {
+                        bpFileItem = $('<li data-filename="'+filename+'"><span><small>'
+                                                +filename+'</small></span><ol></ol></li>')
+                                        .appendTo(breakpointsList);
+                    }
+                    // Since it's a new filename, we can immediately this
+                    // breakpoint to the empty list item
+                    bpFileItem = bpFileItem.children('ol'); // The list inside this list item
+                    bpFileItem.append(newBpListItem);
+
+                } else {
+                    // The file is already listed. Insert the new breakpoint into the
+                    // list at the appropriate place
+                    var bpLineItems = bpFileItem.children(),
+                        nextBpLineItem;
+
+                    for (i = 0; i < bpLineItems.length; i++) {
+                        if (parseInt(bpLineItems[i].attributes.getNamedItem('data-lineno').value) > line) {
+                            nextBpLineItem = bpLineItems[i];
+                            break;
+                        }
+                    }
+                    if (nextBpLineItem) {
+                        newBpListItem.insertBefore(nextBpLineItem);
+                    } else {
+                        newBpListItem.appendTo(bpFileItem);
+                    }
+                }
+            }
+        }
+    };
+    // Triggered by the breakpoint manager whenever a breakpoint changes
+    // Update the line-number icons for this breakpoint
     this.breakpointsForPane = function(e, bp_hash) {
         // bp_hash is a hash of { line: { condition: cond, action: act } }
         // 'this' in this function is the .program-code table with the program code
 
         var $codeTable = $( this ),
+            filename = $codeTable.attr('data-filename'),
             line;
 
         for (line in bp_hash) {
             var condition = bp_hash[line].condition,
+                conditionEnabled = bp_hash[line].conditionEnabled,
                 action = bp_hash[line].action,
                 lineElt = $codeTable.find('.code-line:nth-child('+line+')');
 
             lineElt.removeClass('breakpoint conditional-breakpoint inactive-breakpoint bpaction');
-            if (condition === '0') {
-                lineElt.addClass('inactive-breakpoint');
-            } else if (condition === '1') {
-                lineElt.addClass('breakpoint');
-            } else if (condition) {
-                lineElt.addClass('conditional-breakpoint');
+            if (condition) {
+                if (condition === '1') {
+                    lineElt.addClass('breakpoint');
+                } else {
+                    lineElt.addClass('conditional-breakpoint');
+                }
+                if (! conditionEnabled) {
+                    lineElt.addClass('inactive-breakpoint');
+                }
             }
             if (action) {
                 lineElt.addClass('bpaction');
@@ -219,18 +332,17 @@ function Debugger(sel) {
         var $elt = $(e.target);
 
         e.preventDefault();
-        this._drawBreakpointMenu($elt);
+        this._drawBreakpointRightClickMenu($elt);
     };
-    this._drawBreakpointMenu = function($elt) {
-        var filename = $elt.closest('.program-code').attr('data-filename'),
-            lineno = $elt.closest('.code-line').attr('data-line'),
+    this._drawBreakpointRightClickMenu = function($elt) {
+        var filename = $elt.closest('[data-filename]').attr('data-filename'),
+            lineno = $elt.closest('[data-lineno]').attr('data-lineno'),
             bp = this.breakpointManager.get(filename, lineno),
-            menu = this.breakpointRightClickMenuTemplate({
-                                                    condition: bp.condition,
-                                                    action: bp.action,
-                                                    filename: filename,
-                                                    lineno: lineno
-                                                });
+            menu;
+
+        $.extend(bp, { filename: filename, lineno: lineno });
+        menu = this.breakpointRightClickMenuTemplate($.extend({ filename: filename, lineno: lineno }, bp));
+
         if (this.breakpointPopover) {
             this.breakpointPopover.popover('destroy');
         }
@@ -242,12 +354,67 @@ function Debugger(sel) {
                                                 content: menu})
                                         .popover('show');
     };
+
+    // When the user clicks the checkbox next to a breakpoint condition or action
+    this.toggleBreakpointOrAction = function(e) {
+        var $elt = $(e.currentTarget),
+            filename = $elt.closest('[data-filename]').attr('data-filename'),
+            lineno = $elt.closest('[data-lineno]').attr('data-lineno'),
+            type = $elt.hasClass('toggle-breakpoint') ? 'condition' : 'action',
+            state = $elt.is(':checked'),
+            requestData = { f: filename, l: lineno };
+
+        if (type === 'condition') {
+            // ci means condition inactive, so we need to flip the state value to send
+            requestData.ci = state ? 0 : 1;
+        } else {
+            requestData.ai = state ? 0 : 1;
+        }
+
+        $.ajax({url: 'breakpoint',
+                type: 'POST',
+                dataType: 'json',
+                data: requestData,
+                success: this.messageHandler.consumer,
+                error: this.ajaxError
+            });
+    };
+
+    // When the user clicks the X to remove an item from the breakpoint list
+    this.removeBreakpointFromList = function(e) {
+        var $elt = $(e.currentTarget),
+            filename = $elt.closest('[data-filename]').attr('data-filename'),
+            lineno = $elt.closest('[data-lineno]').attr('data-lineno');
+
+        $.ajax({url: 'breakpoint',
+                type: 'POST',
+                dataType: 'json',
+                data: { f: filename, l: lineno, c: '', a: '' },
+                success: this.messageHandler.consumer,
+                error: this.ajaxError
+            });
+    };
+
+    // Called when the user clicks the swoopy aarow next to a breakpoint location
+    // in the breakpoint pane list
+    this.scrollToBreakpoint = function(e) {
+        var $elt = $(e.currentTarget),
+            filename = $elt.closest('[data-filename]').attr('data-filename'),
+            lineno = $elt.closest('[data-lineno]').attr('data-lineno'),
+            codeTable = this.elt.find('.program-code[data-filename="'+filename+'"]').first();
+
+        var lineElt = this._scrollCodeTableLineIntoView(codeTable, lineno);
+        var originalBg = lineElt.css('background-color');
+        lineElt.css({ 'background-color': 'yellow'});
+        setTimeout(function() { lineElt.css({'background-color': originalBg}) }, 1000);
+    };
+
     // When the user clicks a breakpoint condition or action in the
-    // breakpoint context menu
+    // breakpoint context menu or breakpoint pane list
     this.editBreakpointCondition = function (e) {
         var $elt = $(e.currentTarget),
-            filename = $elt.attr('data-filename'),
-            lineno = $elt.attr('data-lineno'),
+            filename = $elt.closest('[data-filename]').attr('data-filename'),
+            lineno = $elt.closest('[data-lineno]').attr('data-lineno'),
             type = $elt.hasClass('breakpoint-condition') ? 'condition' : 'action',
             dbg = this;
 
@@ -257,6 +424,7 @@ function Debugger(sel) {
             .submit(function(e) {
                 var form = e.target,
                     value = form.elements['input'].value;
+
                 e.preventDefault();
                 if (type === 'condition') {
                     dbg.breakpointManager.set({ filename: filename, lineno: lineno, condition: value});
@@ -264,7 +432,10 @@ function Debugger(sel) {
                 } else if (type === 'action') {
                     dbg.breakpointManager.set({ filename: filename, lineno: lineno, action: value});
                 }
-                dbg._drawBreakpointMenu(dbg.breakpointPopover);
+                if (dbg.breakpointPopover && ($elt.closest('.popover-content').length !== 0)) {
+                    // It was part of the right-click breakpoint menu
+                    dbg._drawBreakpointRightClickMenu(dbg.breakpointPopover);
+                }
             });
     };
 
@@ -391,8 +562,17 @@ function Debugger(sel) {
     this.setCurrentLineForCodeTable = function(codeTable, line) {
         codeTable.find('.active').removeClass('active');
         var activeLine = codeTable.find('.code-line:nth-child('+line+')').addClass('active');
+
+        this._scrollCodeTableLineIntoView(codeTable, line);
+    };
+
+    // Ensure the given line number of the codeTable is visible
+    this._scrollCodeTableLineIntoView = function(codeTable, line) {
+        var activeLine = codeTable.find('.code-line:nth-child('+line+')');
+
         if (activeLine.length === 0) {
-            // When the program terminates, line is 0 and activeLine.length === 0
+            // Bad info.  This can happen when the program terminates, line is
+            // 0 and activeLine.length === 0
             return;
         }
 
@@ -407,8 +587,12 @@ function Debugger(sel) {
         var isVisible = ((elemBottom >= containerTop) && (elemTop <= containerBottom)
                         && (elemBottom <= containerBottom) &&  (elemTop >= containerTop) );
         if (!isVisible) {
-            container.scrollTo(activeLine, { over: -2 });  // Leave 2 lines at the top
+            // If the target element is off the top, leave 2 lines at the top,
+            // if it's off the bottom, leave 2 lines at the bottom
+            var overShoot = (elemBottom < containerTop) ? -2 : 2;
+            container.scrollTo(activeLine, { over: overShoot });
         }
+        return activeLine;
     };
 
     // Called by the stackManager when the given stack frame # has changed from
@@ -620,18 +804,23 @@ function Debugger(sel) {
         this.breakpointUpdated = function(data) {
             var filename    = data.filename,
                 line        = data.lineno,
-                condition   = data.condition;
-                action      = data.action;
+                condition   = data.condition,
+                action      = data.action,
+                conditionInactive   = data.condition_inactive,
+                actionInactive      = data.action_inactive;
 
             var breakpoint = {};
             breakpoint[line] = {};
 
             if (condition) {
                 breakpoint[line].condition = condition;
+                breakpoint[line].conditionEnabled = ! conditionInactive;
             }
             if (action) {
                 breakpoint[line].action = action;
+                breakpoint[line].actionEnabled = ! actionInactive;
             }
+
             if (condition || action) {
                 // breakpoint in a new file
                 if (! (filename in this.breakpoints)) {
@@ -643,7 +832,7 @@ function Debugger(sel) {
                 delete this.breakpoints[filename][line];
             }
 
-
+            dbg.updateBreakpointList(filename, breakpoint);
             $('.program-code[data-filename="'+filename+'"]')
                 .trigger('breakpoints-updated', [ breakpoint ]);
         };
@@ -682,10 +871,26 @@ function Debugger(sel) {
             if ('condition' in params) {
                 this.breakpoints[filename][lineno].condition = params.condition;
                 postData.c = params.condition;
+                if (! ('conditionEnabled' in params)) {
+                    // If enable/disable wasn't given, assume enabled
+                    this.breakpoints[filename][lineno].conditionEnabled = true;
+                }
+            }
+            if ('conditionEnabled' in params) {
+                this.breakpoints[filename][lineno].conditionEnabled = params.conditionEnabled;
+                postData.ci = ! params.conditionEnabled;
             }
             if ('action' in params) {
                 this.breakpoints[filename][lineno].action = params.action;
                 postData.a = params.action;
+                if (! ('actionEnabled' in params)) {
+                    // If enable/disable wasn't given, assume enabled
+                    this.breakpoints[filename][lineno].actionEnabled = true;
+                }
+            }
+            if ('actionEnabled' in params) {
+                this.breakpoints[filename][lineno].actionEnabled = params.actionEnabled;
+                postData.ci = ! params.actionEnabled;
             }
 
             $.ajax({url: 'breakpoint',
