@@ -24,7 +24,7 @@ PerlValue.exception = function(value) {
     this.type = 'exception';
 };
 PerlValue.exception.prototype = new PerlValue();
-PerlValue.exception.prototype.render = function() {
+PerlValue.exception.prototype.renderValue = function() {
     return '<span class="label label-important PerlValue">'+this.value+'</span>';
 };
 
@@ -34,7 +34,7 @@ PerlValue.scalar = function(value) {
     this.type = 'scalar';
 }
 PerlValue.scalar.prototype = new PerlValue();
-PerlValue.scalar.prototype.render = function() {
+PerlValue.scalar.prototype.renderValue = function() {
     if ((this.value === undefined) || (this.value === null)) {
         return '<span class="label PerlValue"><i>undef</i></span>';
     } else {
@@ -50,12 +50,11 @@ PerlValue.SCALAR = function(value) {
     this.blessed = value.__blessed;
 }
 PerlValue.SCALAR.prototype = new PerlValue();
-PerlValue.SCALAR.prototype.render = function() {
-    return '<span>'
-            + this.refHeaderString()
-            + '</span><span class="PerlValue">'
-            + this.value.render()
-            + '</span>';
+PerlValue.SCALAR.prototype.renderHeader = function() {
+    return '<span>' + this.refHeaderString() + '</span>';
+};
+PerlValue.SCALAR.prototype.renderValue = function() {
+    return '<span class="PerlValue">' + this.value.render() + '</span>';
 }
 
 // ARRAY ref values
@@ -77,7 +76,7 @@ PerlValue.ARRAY.prototype.renderHeader = function() {
             + this.count +'</span>';
 }
 PerlValue.ARRAY.prototype.renderValue = function() {
-    var html = '<dl>';
+    var html = '<dl class="PerlValue">';
     for(var i = 0; i < this.value.length; i++) {
         html += '<dt>' + i + '</dt><dd>'
                 + this.value[i].render() + '</dd>';
@@ -108,7 +107,7 @@ PerlValue.HASH.prototype.renderHeader = function() {
             + this.count + '</span>';
 }
 PerlValue.HASH.prototype.renderValue = function() {
-    var html = '<dl>',
+    var html = '<dl class="PerlValue">',
         k;
     for (key in this.value) {
         html += '<dt>' + key + '</dt><dd>' + this.value[key].render() + '</dd>';
@@ -136,7 +135,7 @@ PerlValue.GLOB.prototype.renderHeader = function() {
             + ' <span class="badge badge-info expr-collapse-button">&plusmn;</span>';
 }
 PerlValue.GLOB.prototype.renderValue = function() {
-    var html = '<dl>',
+    var html = '<dl class="PerlValue">',
         k;
     for (k in this.value) {
         html += '<dt><span class="label label-success">' + k + '</span></dt><dd>'
@@ -170,7 +169,18 @@ PerlValue.REF.prototype.renderHeader = function() {
             + '<span class="badge badge-info expr-collapse-button">&plusmn;</span>';
 }
 PerlValue.REF.prototype.renderValue = function() {
-    return '<ul><li>' + this.value.render() + '</li></ul>';
+    return '<ul class="PerlValue"><li>' + this.value.render() + '</li></ul>';
+}
+
+//Regexp (from qr() ) values
+PerlValue.REGEXP = function(value) {
+    this.value = value.__value;
+    this.refaddr = value.__refaddr;
+    this.type = value.__reftype;
+    this.blessed = value.__blessed;
+}
+PerlValue.REGEXP.prototype.renderValue = function() {
+    return '<span class="PerlValue">' + this.value + '</span>';
 }
 
 
@@ -180,7 +190,9 @@ PerlValue.prototype.refHeaderString = function() {
     var html = '';
     if (this.blessed) {
         html += this.blessed + '=';
-    }
+    0}
     html += this.type + '(0x' + this.refaddr.toString(16) + ')';
     return html;
 }
+PerlValue.prototype.renderHeader = function() { return '' };
+PerlValue.prototype.renderValue = function() { return '' };
